# begin PrettyPrinter.py
#
# By Michael Scharf. 
# Michael_Scharf@TakeFive.co.at or Michael.Scharf@EMBL-Heidelberg.de
# NO WARRANTY!
# Do whatever you want with it!
"""Pretty print nested data structures.

Exports a class PrettyPrinter and a function Print.

"""


from types import TupleType,ListType,DictType,InstanceType

recursiveTypes=(DictType,ListType,InstanceType)

class PrettyPrinter:
    def __init__(self,
                file      = None,
                indent    = None,
                separator = None,
                depth     = None,
                duplicate = None):
        if indent is None:
            indent='    '
        self.indent=indent
        if file is None:
            import sys
            file=sys.stdout
        self.file=file
        if separator is None:
            separator=''
        self.separator=separator
        self.depth=depth
        self.duplicate=duplicate

    def write(self,str):
        self.file.write(str)

    def nl(self,depth):
        self.write('\n'+ self.indent * depth)

    def GoDeeper(self,depth):
        """Check is we are too deep in the recursion"""
        if self.depth is not None and depth>=self.depth:
            self.write('...')
            return 0
        return 1            
        
    def Print(self,value):
        self.seen={} # helper instance to avoid infinite recursion
        self.HandleAny(value)
        self.nl(0)
        del self.seen

    def HandleList(self,value,depth=0,nl=0):
        self.write('[')
        separator=''
        if self.GoDeeper(depth):
            for v in value:
                self.write(separator)
                self.HandleAny(v,depth+1,1)
                separator=self.separator
            if len(value):
                self.nl(depth)
        self.write(']')

    def HandleTuple(self,value,depth=0,nl=0):
        self.write('(')
        separator=''
        if self.GoDeeper(depth):
            for v in value:
                self.write(separator)
                self.HandleAny(v,depth+1,1)
                separator=self.separator
            if len(value):
                self.nl(depth)
        self.write(')')

    def HandleDict(self,value,depth=0,nl=0):
        self.write('{')
        separator=''
        if self.GoDeeper(depth):
            for k,v in value.items():
                self.write(separator)
                self.HandleAny(k,depth+1,1)
                self.write(" : ")
                self.HandleAny(v,depth+1,0)
                separator=self.separator
            if len(value):
                self.nl(depth)
        self.write('}')

    def HandleInstance(self,value,depth=0,nl=0):
        self.write(value.__class__.__name__)
        self.write('(')
        dict=value.__dict__
        # find the length of the longest name
        l=max(map(len,dict.keys()))
        # build a format string lile "%11s"
        format='%%-%ds' %l
        separator=''
        if self.GoDeeper(depth):
            for k,v in dict.items():
                self.write(separator)
                self.nl(depth+1)
                self.write(format % k)
                self.write(" = ")
                self.HandleAny(v,depth+1,0)
                separator=self.separator
            if len(dict):
                self.nl(depth)
        self.write(')')

    def HandleOther(self,value,depth=0,nl=0):
        self.write(`value`)

    def Register(self,value):
        seen=0
        if type(value) in recursiveTypes:
            key=id(value)
            if self.seen.has_key(key):
                # don't print ourself again
                seen=1
            self.seen[key]=None
        return seen

    def Unregister(self,value):
        if type(value) in recursiveTypes:
            key=id(value)
            del self.seen[key]

    def HandleAny(self,value,depth=0,nl=0):
        if nl:
            self.nl(depth)
        if self.Register(value):
            self.write('***RECURSION***')
            return
        t=type(value)
        if t is TupleType:
            self.HandleTuple(value,depth,nl)
        elif t is ListType:
            self.HandleList(value,depth,nl)
        elif t is DictType:
            self.HandleDict(value,depth,nl)
        elif t is InstanceType:
            self.HandleInstance(value,depth,nl)
        else:
            self.HandleOther(value,depth,nl)
        if self.duplicate:
            # we want to print this value again
            self.Unregister(value)

def Print(value,file=None,indent=None,
          separator=None,depth=None,duplicate=None):
    p=PrettyPrinter(file,indent,separator,depth,duplicate)
    p.Print(value)

def _test():
    class Person:
        def __init__(self,**attrs):
            for name,val in attrs.items():
                setattr(self,name,val)
    
    guido=Person(
        Name='Guido',
        SirName='van Rossum',
        Language='python',  
        Keywords=('for','class','and','def')
    )
    larry=Person(
        Name='Larry', 
        SirName='Wall',
        Language='perl',
        Keywords=['&','$','@','%','/']
    )
    # let's create a recursive data structure
    guido.Competitor=larry
    larry.Competitor=guido
    # and a list with two guidos
    data=(guido,larry,guido,{'dict':{},'list':[1,2]})
    Print(data,duplicate=1)
    Print(data,indent='  ',separator=',',depth=3)
    guido.Competitor=None # remove larry to stop the recussion ;-)
    # how does a pretty printer look like??
    pp=PrettyPrinter(indent=' ',separator=';')
    pp.Print(pp)
        
if __name__=='__main__':
    _test()

# end PrettyPrinter.py
